/* crde.h - Copyright (c) 1990, 1991, Mark Lang */

/* The C Relational Database Engine 2.00. 	*/
/* For Turbo C 2.0 and C++ 1.0 compilers.	*/
/* All Rights Reserved.				*/

#ifndef __CRDE__
#define __CRDE__

#ifdef  __STDC__
#define tdecl(type) type
#else
#define tdecl(type) type cdecl
#endif

/* TABLE and CURSOR decls */

typedef void TABLE;
typedef void CURSOR;

/* date_t type defined */

typedef long date_t;

/* initialization modes */

#define NET_NONE	0
#define NET_MSDOS	1
#define NET_NOVELL	2

/* table limits */

#define MAXTABLES     	128
#define MAXTABLENAME  	255
#define MAXROWSIZE    	4000
#define MAXCOLNAME    	16
#define MAXCOLSIZE    	255
#define MAXKEYCOLS    	8
#define MAXKEYSIZE    	255
#define MAXROWS       	2147483647
#define MAXCOLS       	255
#define MAXINDEXES    	8
#define MAXJOINCOLS   	255
#define MAXEXPR       	16
#define MAXCHANGES 	16
#define MAXPASSNAME	16

/* field types */

#define TSTRING		0
#define TCHAR		1
#define TUCHAR		2
#define TSHORT		3
#define TUSHORT		4
#define TINT		5
#define TUINT		6
#define TLONG		7
#define TULONG		8
#define TFLOAT		9
#define TDOUB		10
#define TLDOUB		11
#define TDATE		12

/* search relops */

#define EQ  		0
#define LT  		1
#define GT  		2
#define LE  		3
#define GE  		4
#define NE  		5

#define LTE 		LE
#define GTE 		GE
#define NEQ 		NE

/* temporary table id's */

#define AUTO    	((char *)0L)
#define STATIC  	((char *)1L)
#define TEMP		AUTO

/* table modes */

#define T_NORMAL 	0
#define T_WRITEC 	1
#define T_AUTO   	2
#define T_STATIC 	3
#define T_TEMP		T_AUTO

/* table locks */

#define TL_READ		0
#define TL_WRITE	1
#define TL_SHARE	2
#define TL_EXCL         3

/* cursor functions */

#define TC_RESET	0
#define TC_NEXT		1
#define TC_PREV		2
#define TC_FIND		3
#define TC_POS		4
#define TC_GOTO		5
#define TC_GET		6
#define TC_PUT		7
#define TC_INSERT	8
#define TC_DELETE	9
#define TC_LOCK		10
#define TC_UNLOCK	11
#define TC_STAT		12
#define TC_DROP		13

/* data structures */

typedef struct {
  char  name[MAXCOLNAME];
  char  type;
  char  pkey;
  short offset;
  short size;
} FIELD;

typedef struct {
  char  keyno;
  char  desc;
  short colno;
} KEY;

typedef struct {
  short colno;
  short relop;
  union {
    char 	  *cp;
    char            c;
    unsigned char  uc;
    short	    s;
    unsigned short us;
    int		    i;
    unsigned int   ui;
    long	    l;
    unsigned long  ul;
    float     	    f;
    double	    d;
    long double	   ld;
  } value;
} CONDITION;

typedef struct {
  char sig;
  char pos;
  void *rec;
} tc_find;

typedef struct {
  long  pos;
  char  indexed;
  char  locked;
  TABLE *t;
} tc_stat;

/* function prototypes */

typedef tdecl(int) iffunc(void *);
typedef tdecl(int) groupfunc(void *, void *, int);

#ifdef __cplusplus
extern "C" {
#endif

/* date functions */

tdecl(int)    	day        	(date_t dt);
tdecl(int)    	dayofweek  	(date_t dt);
tdecl(int)    	dayofyear  	(int month, int day, int year);
tdecl(int)    	daysinmonth	(int month, int year);
tdecl(int)    	daysinyear 	(int year);
tdecl(date_t) 	firstday   	(int month, int year, int dayofweek);
tdecl(int)    	gmdate     	(date_t dt, int *month, int *day, int *year);
tdecl(int)    	leapyear   	(int year);
tdecl(date_t) 	mkdate     	(int month, int day, int year);
tdecl(int)    	month      	(date_t dt);
tdecl(date_t) 	monthadd   	(date_t dt, int months);
tdecl(int)    	monthday   	(int days, int year, int *month, int *day);
tdecl(int)    	year       	(date_t dt);
tdecl(date_t) 	yearadd    	(date_t dt, int years);

/* table functions */

tdecl(int) 	taccess     	(char *name, int mode);
tdecl(long)	tadd        	(TABLE *t1, TABLE *t2);
tdecl(int)	tall        	(tdecl(int) (*func)(TABLE *));
tdecl(int)	taverage    	(TABLE *t, char *c, void *result);
tdecl(TABLE *)	tborrow     	(char *name, TABLE *t);
tdecl(long)	tchange     	(TABLE *t, ...);
tdecl(long)	tchangeif   	(TABLE *t, iffunc *action, ...);
tdecl(long)	tchange2	(TABLE *t, iffunc *action, CONDITION *changes,
				int clen, CONDITION *sexpr, int slen);
tdecl(int)	tchanged	(TABLE *t);
tdecl(int)	tchcol      	(TABLE *t, char *oldcol, char *newcol);
tdecl(int)	tclose      	(TABLE *t);
tdecl(int)	tcmpstruct  	(TABLE *t1, TABLE *t2);
tdecl(int)	tcntl		(TABLE *t, int ctl, ...);
tdecl(char *)	tcolname	(TABLE *t, int n, char *buf);
tdecl(int)	tcolno		(TABLE *t, char *c);
tdecl(int) 	tcols       	(TABLE *t);
tdecl(int)	tcolsize    	(TABLE *t, char *c);
tdecl(int)	tcoltype    	(TABLE *t, char *c);
tdecl(int) 	tcommit     	(TABLE *t);
tdecl(TABLE *)	tcopy       	(char *name, TABLE *t);
tdecl(int) 	tcount      	(TABLE *t, char *c, void *result);
tdecl(TABLE *)	tcreat      	(char *name, char *td);
tdecl(TABLE *)	tcreat2		(char *name, FIELD *td, int n);
tdecl(TABLE *)	tcrstruct   	(char *name, TABLE *t);
tdecl(int)	tcrypt	   	(TABLE *t, char *passwd);
tdecl(int)	tcursor		(TABLE *t, int ctl, ...);
tdecl(long)	tdelete     	(TABLE *t, ...);
tdecl(long)	tdeleteif   	(TABLE *t, iffunc *action, ...);
tdecl(long)	tdelete2	(TABLE *t, iffunc *action, CONDITION *sexpr,
				int len);
tdecl(TABLE *)	tdiff       	(char *name, TABLE *t1, TABLE *t2);
tdecl(int) 	tdrop       	(TABLE *t);
tdecl(int)	tdropcursor	(CURSOR *c);
tdecl(int) 	tdropindex  	(TABLE *t, char *id);
tdecl(int) 	tempty      	(TABLE *t);
tdecl(int) 	terase      	(char *name);
tdecl(long) 	texportascii	(TABLE *t, char *dest);
tdecl(long)	texportdBASE	(TABLE *t, char *dest);
tdecl(int) 	tflush      	(TABLE *t);
tdecl(long)	tget        	(TABLE *t, void *buf, long n, ...);
tdecl(long)	tgetif      	(TABLE *t, void *buf, long n, iffunc *action,
				...);
tdecl(long)	tget2		(TABLE *t, void *buf, long n, iffunc *action,
				CONDITION *sexpr, int len);
tdecl(TABLE *)	tgroup      	(char *name, TABLE *t, char *id, char *td,
				groupfunc *summary);
tdecl(TABLE *)	timportascii	(char *name, char *td, char *source);
tdecl(TABLE *)	timportdBASE	(char *name, char *source);
tdecl(int) 	tindex      	(TABLE *t, char *id);
tdecl(int) 	tindexes    	(TABLE *t);
tdecl(int)	tinit		(int mode, char *privdir, int bufs);
tdecl(long) 	tinsert     	(TABLE *t, void *recs, long n);
tdecl(TABLE *)	tintersect  	(char *name, TABLE *t1, TABLE *t2);
tdecl(TABLE *)	tjoin       	(char *name, TABLE *t1, char *cl1, TABLE *t2,
				char *cl2, char *pl);
tdecl(int)	tkeyed		(TABLE *t);
tdecl(TABLE *)	tkeys		(char *name, TABLE *t);
tdecl(int)	tkeys2		(KEY *k, TABLE *t);
tdecl(int)	tload       	(TABLE *t, ...);
tdecl(int)	tlock		(TABLE *t, int lock);
tdecl(int) 	tlookup     	(TABLE *t, ...);
tdecl(int)	tlookupif   	(TABLE *t, iffunc *action, ...);
tdecl(int)	tlookup2	(TABLE *t, iffunc *action, CONDITION *sexpr,
				int len);
tdecl(int) 	tmark       	(TABLE *t);
tdecl(int)	tmax        	(TABLE *t, char *c, void *result);
tdecl(int)	tmin        	(TABLE *t, char *c, void *result);
tdecl(int) 	tmode       	(TABLE *t);
tdecl(int)	tnormal     	(TABLE *t);
tdecl(TABLE *)	topen       	(char *name, ...);
tdecl(CURSOR *)	topencursor	(TABLE *t, char *id, ...);
tdecl(TABLE *)	tproject    	(char *name, TABLE *t, char *pl, ...);
tdecl(TABLE *)	tprojectif  	(char *name, TABLE *t, char *pl,
				iffunc *action, ...);
tdecl(TABLE *)	tproject2	(char *name, TABLE *t, char *pl,
				iffunc *action, CONDITION *sexpr, int len);
tdecl(int)	tquit		(void);
tdecl(int)	trelease    	(TABLE *t);
tdecl(int)	trename     	(char *oldname, char *newname);
tdecl(TABLE *)	trepair     	(char *name, char *td);
tdecl(long) 	treplace    	(TABLE *t, void *recs, long n);
tdecl(TABLE *)	trestruct   	(char *name, TABLE *t, char *td);
tdecl(int) 	trollback   	(TABLE *t);
tdecl(long)	trows       	(TABLE *t);
tdecl(int) 	trowsize    	(TABLE *t);
tdecl(long)	tscan       	(TABLE *t, ...);
tdecl(long) 	tscanif     	(TABLE *t, iffunc *action, ...);
tdecl(long)	tscan2		(TABLE *t, iffunc *action, CONDITION *sexpr,
				int len);
tdecl(TABLE *)	tselect     	(char *name, TABLE *t, ...);
tdecl(TABLE *)	tselectif   	(char *name, TABLE *t, iffunc *action, ...);
tdecl(TABLE *)	tselect2   	(char *name, TABLE *t, iffunc *action,
				CONDITION *sexpr, int len);
tdecl(TABLE *)	tsort  		(char *name, TABLE *t, char *id);
tdecl(TABLE *)	tstruct		(char *name, TABLE *t);
tdecl(int)	tstruct2	(FIELD *f, TABLE *t);
tdecl(long) 	tsubtract	(TABLE *t1, TABLE *t2);
tdecl(int) 	tsum		(TABLE *t, char *c, void *result);
tdecl(int) 	ttransact	(TABLE *t);
tdecl(int)	tuncrypt	(TABLE *t);
tdecl(TABLE *)	tunion      	(char *name, TABLE *t1, TABLE *t2);
tdecl(int)	tunlock		(TABLE *t);
tdecl(long)	tupdate     	(TABLE *t1, TABLE *t2);
tdecl(int)	tview       	(TABLE *t, char *cl, char *id, int ax, int ay,
				int bx, int by, int cattr, int fattr,
				int battr, void *rec);
tdecl(int)	tver		(char *buf);
tdecl(int) 	twritec     	(TABLE *t);

#ifdef __cplusplus
}
#endif

/* global variables */

tdecl(extern int)  terrno;            /* contains last error code        */
tdecl(extern long) trowsfound;        /* rows involved in last operation */

/* error codes */

/* low-level errors */

#define TE_OUTOFMEM	-10
#define TE_NOINIT	-11
#define TE_INTEGRITY	-15
#define TE_ACCESS	-16

/* disk IO errors */

#define TE_DISKSEEK	-20
#define TE_DISKREAD 	-21
#define TE_DISKWRITE	-22
#define TE_FILECREAT	-23
#define TE_FILEOPEN  	-24

/* key violation errors */

#define TE_KEYVIOL  	-40
#define TE_KEYNOTFOUND	-41

/* rec violation errors */

#define TE_RECDELETED 	-50

/* parsing errors */

#define TE_BADNAME   	-100
#define TE_BIGTOKEN 	-101
#define TE_EMPTYLIST 	-102
#define TE_LIST2BIG 	-103
#define TE_SYNTAX   	-104

/* limit errors */

#define TE_COL2BIG  	-110
#define TE_KEY2BIG 	-111
#define TE_ROW2BIG 	-112
#define TE_2MANYCOLS 	-113
#define TE_2MANYKEYCOLS	-114
#define TE_2MANYROWS 	-115
#define TE_2MANYINDEXES	-116
#define TE_NOCOLS   	-117
#define TE_2MANYTABLES	-118

/* syntax errors */

#define TE_BADTYPE  	-120
#define TE_BADCOL  	-121
#define TE_BADINDEX  	-122
#define TE_DUPCOL   	-123
#define TE_DUPKEYCOL 	-124

/* system errors */

#define TE_INVALIDTABLE	-130
#define TE_READONLY   	-131
#define TE_TRANSACT  	-132
#define TE_CORRUPTTABLE	-133
#define TE_DUPTABLES  	-134
#define TE_CORRUPTINDEX	-135
#define TE_TEMP     	-136

/* search/change expression errors */

#define TE_RELOP      	-140
#define TE_SEXPRCOMPLEX	-141
#define TE_2MANYCHGS 	-142

/* miscelleneous errors */

#define TE_JOINCOLS 	-150
#define TE_STRUCT  	-151
#define TE_CRSTRUCT  	-152
#define TE_VIEWSPECS	-153
#define TE_NOCONVERT  	-154
#define TE_NULLRESULT	-155

/* networking errors */

#define TE_SHAREVIOL	-170
#define TE_SHAREFAIL 	-171

/* cursor errors */

#define TE_CVALID	-180
#define TE_CBADFUNC	-181
#define TE_CREADONLY	-182
#define TE_CLOCKVIOL	-183
#define TE_CLOCKFAIL	-184
#define TE_CRECVALID	-185
#define TE_CEOF		-186

#endif

/* end of crde.h */


